@@ -77,11 +77,11 @@ class Reader(object):
         message
 
     :param lookupd_poll_interval: the amount of time in seconds between querying
-        all of the supplied nsqlookupd instances.  a random amount of time based
-        on thie value will be initially introduced in order to add jitter when
+        all of the supplied nsqlookupd instances.  A random amount of time based
+        on this value will be initially introduced in order to add jitter when
         multiple readers are running
 
-    :param lookupd_poll_jitter: The maximum fractional amount of jitter to add
+    :param lookupd_poll_jitter: the maximum fractional amount of jitter to add
         to the lookupd pool loop. This helps evenly distribute requests even if
         multiple consumers restart at the same time.
 
@@ -218,8 +218,9 @@ class Reader(object):
         """Emitted after a giving up on a message.
 
         Emitted when a message has exceeded the maximum number of attempts
-        (`max_tries`) and will no longer be requeued. The signal sender is the
-        reader and the `message` is sent as an argument.
+        (`max_tries`) and will no longer be requeued. This is useful to perform
+        tasks such as writing to disk, collecting statistics etc. The signal
+        sender is the reader and the `message` is sent as an argument.
         """
         return blinker.Signal(doc='Sent after a giving up on a message.')
 
@@ -295,8 +296,7 @@ class Reader(object):
         self.state = CLOSED
 
         self.logger.debug('closing %d worker(s)' % len(self.workers))
-        for worker in self.workers:
-            worker.kill(block=False)
+        gevent.killall(self.workers, block=False)
 
         self.logger.debug('closing %d connection(s)' % len(self.conns))
         for conn in self.conns:
@@ -316,11 +316,12 @@ class Reader(object):
 
     @property
     def is_starved(self):
-        """Check if reader is currently starved for messages."""
-        for conn in self.conns:
-            if conn.in_flight >= max(conn.last_ready * 0.85, 1):
-                return True
-        return False
+        """Evaluate whether any of the connections are starved.
+
+        This property should be used by message handlers to reliably identify
+        when to process a batch of messages.
+        """
+        return any(conn.is_starved for conn in self.conns)
 
     @property
     def connection_max_in_flight(self):
@@ -345,11 +346,14 @@ class Reader(object):
         if self.state == THROTTLED and self.total_in_flight_or_ready:
             return
 
-        if (self.total_in_flight_or_ready + count) > self.max_in_flight:
+        total = self.total_ready_count - conn.ready_count + count
+        if total > self.max_in_flight:
             if not (conn.ready_count or conn.in_flight):
+                self.logger.debug('[%s] sending later' % conn)
                 gevent.spawn_later(5, self.send_ready, conn, count)
             return
 
+        self.logger.debug('[%s] sending RDY %d' % (conn, count))
         conn.ready(count)
 
     def query_nsqd(self):
@@ -461,7 +465,7 @@ class Reader(object):
         if self.state == THROTTLED:
             max_in_flight = 1 - self.total_in_flight_or_ready
         else:
-            max_in_flight = self.max_in_flight - self.total_in_flight_or_ready
+            max_in_flight = self.max_in_flight - self.total_ready_count
 
         if max_in_flight <= 0:
             return
@@ -628,6 +632,7 @@ class Reader(object):
 
     def handle_message(self, conn, message):
         self.logger.debug('[%s] got message: %s' % (conn, message.id))
+        self.update_ready(conn)
 
         if self.max_tries and message.attempts > self.max_tries:
             msg = "giving up on message '%s' after max tries %d"
